@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -20,10 +20,10 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-__version__ = '1.2.0'
-__author__ = 'Raphaël Barrois <raphael.barrois@polytechnique.org>'
+__version__ = '1.3.0'
+__author__ = 'Raphaël Barrois <raphael.barrois+fboy@polytechnique.org>'
 
-from base import (
+from .base import (
     Factory,
     StubFactory,
     DjangoModelFactory,
@@ -51,7 +51,7 @@ from base import (
     MOGO_BUILD,
 )
 
-from declarations import (
+from .declarations import (
     LazyAttribute,
     Iterator,
     InfiniteIterator,
@@ -62,6 +62,7 @@ from declarations import (
     SubFactory,
     CircularSubFactory,
     PostGeneration,
+    PostGenerationMethodCall,
     RelatedFactory,
 
     lazy_attribute,
