@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -31,10 +31,17 @@ BUILD_STRATEGY = 'build'
 CREATE_STRATEGY = 'create'
 STUB_STRATEGY = 'stub'
 
-# Creation functions. Use Factory.set_creation_function() to set a creation function appropriate for your ORM.
-DJANGO_CREATION = lambda class_to_create, **kwargs: class_to_create.objects.create(**kwargs)
+# Creation functions. Deprecated.
+# Override Factory._create instead.
+def DJANGO_CREATION(class_to_create, **kwargs):
+    warnings.warn(
+        "Factories defaulting to Django's Foo.objects.create() is deprecated, "
+        "and will be removed in the future. Please inherit from "
+        "factory.DjangoModelFactory instead.", PendingDeprecationWarning, 6)
+    return class_to_create.objects.create(**kwargs)
 
-# Building functions. Use Factory.set_building_function() to set a building functions appropriate for your ORM.
+# Building functions. Deprecated.
+# Override Factory._build instead.
 NAIVE_BUILD = lambda class_to_build, **kwargs: class_to_build(**kwargs)
 MOGO_BUILD = lambda class_to_build, **kwargs: class_to_build.new(**kwargs)
 
@@ -163,7 +170,7 @@ class FactoryMetaClass(BaseFactoryMetaClass):
         if FACTORY_CLASS_DECLARATION in attrs:
             return attrs[FACTORY_CLASS_DECLARATION]
 
-        # No specific associated calss was given, and one was defined for our
+        # No specific associated class was given, and one was defined for our
         # parent, use it.
         if inherited is not None:
             return inherited
@@ -182,7 +189,7 @@ class FactoryMetaClass(BaseFactoryMetaClass):
                             FACTORY_CLASS_DECLARATION,
                             associated_name,
                             class_name,
-                        ), PendingDeprecationWarning)
+                        ), DeprecationWarning, 3)
 
                     return getattr(factory_module, associated_name)
 
@@ -220,9 +227,6 @@ class FactoryMetaClass(BaseFactoryMetaClass):
         associated_class = cls._discover_associated_class(class_name, attrs,
                 inherited_associated_class)
 
-        # Remove the FACTORY_CLASS_DECLARATION attribute from attrs, if present.
-        attrs.pop(FACTORY_CLASS_DECLARATION, None)
-
         # If inheriting the factory from a parent, keep a link to it.
         # This allows to use the sequence counters from the parents.
         if associated_class == inherited_associated_class:
@@ -261,6 +265,9 @@ class BaseFactory(object):
     # class.
     _base_factory = None
 
+    # List of arguments that should be passed as *args instead of **kwargs
+    FACTORY_ARG_PARAMETERS = ()
+
     @classmethod
     def _setup_next_sequence(cls):
         """Set up an initial sequence value for Sequence attributes.
@@ -316,6 +323,36 @@ class BaseFactory(object):
         """
         return getattr(cls, CLASS_ATTRIBUTE_DECLARATIONS).copy(extra_defs)
 
+    @classmethod
+    def _build(cls, target_class, *args, **kwargs):
+        """Actually build an instance of the target_class.
+
+        Customization point, will be called once the full set of args and kwargs
+        has been computed.
+
+        Args:
+            target_class (type): the class for which an instance should be
+                built
+            args (tuple): arguments to use when building the class
+            kwargs (dict): keyword arguments to use when building the class
+        """
+        return target_class(*args, **kwargs)
+
+    @classmethod
+    def _create(cls, target_class, *args, **kwargs):
+        """Actually create an instance of the target_class.
+
+        Customization point, will be called once the full set of args and kwargs
+        has been computed.
+
+        Args:
+            target_class (type): the class for which an instance should be
+                created
+            args (tuple): arguments to use when creating the class
+            kwargs (dict): keyword arguments to use when creating the class
+        """
+        return target_class(*args, **kwargs)
+
     @classmethod
     def build(cls, **kwargs):
         """Build an instance of the associated class, with overriden attrs."""
@@ -331,7 +368,7 @@ class BaseFactory(object):
         Returns:
             object list: the built instances
         """
-        return [cls.build(**kwargs) for _ in xrange(size)]
+        return [cls.build(**kwargs) for _ in range(size)]
 
     @classmethod
     def create(cls, **kwargs):
@@ -348,7 +385,7 @@ class BaseFactory(object):
         Returns:
             object list: the created instances
         """
-        return [cls.create(**kwargs) for _ in xrange(size)]
+        return [cls.create(**kwargs) for _ in range(size)]
 
     @classmethod
     def stub(cls, **kwargs):
@@ -358,7 +395,7 @@ class BaseFactory(object):
         factory's declarations or in the extra kwargs.
         """
         stub_object = containers.StubObject()
-        for name, value in cls.attributes(create=False, extra=kwargs).iteritems():
+        for name, value in cls.attributes(create=False, extra=kwargs).items():
             setattr(stub_object, name, value)
         return stub_object
 
@@ -372,7 +409,7 @@ class BaseFactory(object):
         Returns:
             object list: the stubbed instances
         """
-        return [cls.stub(**kwargs) for _ in xrange(size)]
+        return [cls.stub(**kwargs) for _ in range(size)]
 
     @classmethod
     def generate(cls, strategy, **kwargs):
@@ -460,10 +497,6 @@ class Factory(BaseFactory):
     class AssociatedClassError(RuntimeError):
         pass
 
-    # Customizing 'create' strategy, using a tuple to keep the creation function
-    # from turning it into an instance method.
-    _creation_function = (DJANGO_CREATION,)
-
     @classmethod
     def set_creation_function(cls, creation_function):
         """Set the creation function for this class.
@@ -474,6 +507,12 @@ class Factory(BaseFactory):
                 which an instance will be created. The value of the various
                 fields are passed as keyword arguments.
         """
+        warnings.warn(
+            "Use of factory.set_creation_function is deprecated, and will be "
+            "removed in the future. Please override Factory._create() instead.",
+            PendingDeprecationWarning, 2)
+        # Customizing 'create' strategy, using a tuple to keep the creation function
+        # from turning it into an instance method.
         cls._creation_function = (creation_function,)
 
     @classmethod
@@ -485,11 +524,17 @@ class Factory(BaseFactory):
                 an instance will be created, and keyword arguments for the value
                 of the fields of the instance.
         """
-        return cls._creation_function[0]
-
-    # Customizing 'build' strategy, using a tuple to keep the creation function
-    # from turning it into an instance method.
-    _building_function = (NAIVE_BUILD,)
+        creation_function = getattr(cls, '_creation_function', ())
+        if creation_function and creation_function[0]:
+            return creation_function[0]
+        elif cls._create.__func__ == Factory._create.__func__ and \
+                hasattr(cls._associated_class, 'objects'):
+            # Backwards compatibility.
+            # Default creation_function and default _create() behavior.
+            # The best "Vanilla" _create detection algorithm I found is relying
+            # on actual method implementation (otherwise, make_factory isn't
+            # detected as 'default').
+            return DJANGO_CREATION
 
     @classmethod
     def set_building_function(cls, building_function):
@@ -501,6 +546,12 @@ class Factory(BaseFactory):
                 which an instance will be built. The value of the various
                 fields are passed as keyword arguments.
         """
+        warnings.warn(
+            "Use of factory.set_building_function is deprecated, and will be "
+            "removed in the future. Please override Factory._build() instead.",
+            PendingDeprecationWarning, 2)
+        # Customizing 'build' strategy, using a tuple to keep the creation function
+        # from turning it into an instance method.
         cls._building_function = (building_function,)
 
     @classmethod
@@ -512,7 +563,14 @@ class Factory(BaseFactory):
                 an instance will be created, and keyword arguments for the value
                 of the fields of the instance.
         """
-        return cls._building_function[0]
+        building_function = getattr(cls, '_building_function', ())
+        if building_function and building_function[0]:
+            return building_function[0]
+
+    @classmethod
+    def _adjust_kwargs(cls, **kwargs):
+        """Extension point for custom kwargs adjustment."""
+        return kwargs
 
     @classmethod
     def _prepare(cls, create, **kwargs):
@@ -522,10 +580,24 @@ class Factory(BaseFactory):
             create: bool, whether to create or to build the object
             **kwargs: arguments to pass to the creation function
         """
+        target_class = getattr(cls, CLASS_ATTRIBUTE_ASSOCIATED_CLASS)
+        kwargs = cls._adjust_kwargs(**kwargs)
+
+        # Extract *args from **kwargs
+        args = tuple(kwargs.pop(key) for key in cls.FACTORY_ARG_PARAMETERS)
+
         if create:
-            return cls.get_creation_function()(getattr(cls, CLASS_ATTRIBUTE_ASSOCIATED_CLASS), **kwargs)
+            # Backwards compatibility
+            creation_function = cls.get_creation_function()
+            if creation_function:
+                return creation_function(target_class, *args, **kwargs)
+            return cls._create(target_class, *args, **kwargs)
         else:
-            return cls.get_building_function()(getattr(cls, CLASS_ATTRIBUTE_ASSOCIATED_CLASS), **kwargs)
+            # Backwards compatibility
+            building_function = cls.get_building_function()
+            if building_function:
+                return building_function(target_class, *args, **kwargs)
+            return cls._build(target_class, *args, **kwargs)
 
     @classmethod
     def _generate(cls, create, attrs):
@@ -545,11 +617,26 @@ class Factory(BaseFactory):
         obj = cls._prepare(create, **attrs)
 
         # Handle post-generation attributes
+        results = {}
         for name, decl in sorted(postgen_declarations.items()):
             extracted, extracted_kwargs = postgen_attributes[name]
-            decl.call(obj, create, extracted, **extracted_kwargs)
+            results[name] = decl.call(obj, create, extracted, **extracted_kwargs)
+
+        cls._after_postgeneration(obj, create, results)
+
         return obj
 
+    @classmethod
+    def _after_postgeneration(cls, obj, create, results=None):
+        """Hook called after post-generation declarations have been handled.
+
+        Args:
+            obj (object): the generated object
+            create (bool): whether the strategy was 'build' or 'create'
+            results (dict or None): result of post-generation declarations
+        """
+        pass
+
     @classmethod
     def build(cls, **kwargs):
         attrs = cls.attributes(create=False, extra=kwargs)
@@ -574,13 +661,34 @@ class DjangoModelFactory(Factory):
 
     @classmethod
     def _setup_next_sequence(cls):
-        """Compute the next available ID, based on the 'id' database field."""
+        """Compute the next available PK, based on the 'pk' database field."""
         try:
-            return 1 + cls._associated_class._default_manager.values_list('id', flat=True
-                ).order_by('-id')[0]
+            return 1 + cls._associated_class._default_manager.values_list('pk', flat=True
+                ).order_by('-pk')[0]
         except IndexError:
             return 1
 
+    @classmethod
+    def _create(cls, target_class, *args, **kwargs):
+        """Create an instance of the model, and save it to the database."""
+        return target_class._default_manager.create(*args, **kwargs)
+
+    @classmethod
+    def _after_postgeneration(cls, obj, create, results=None):
+        """Save again the instance if creating and at least one hook ran."""
+        if create and results:
+            # Some post-generation hooks ran, and may have modified us.
+            obj.save()
+
+
+class MogoFactory(Factory):
+    """Factory for mogo objects."""
+    ABSTRACT_FACTORY = True
+
+    @classmethod
+    def _build(cls, target_class, *args, **kwargs):
+        return target_class.new(*args, **kwargs)
+
 
 def make_factory(klass, **kwargs):
     """Create a new, simple factory for the given class."""
