@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2010 Mark Sandstrom
-# Copyright (c) 2011 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,9 @@
 # THE SOFTWARE.
 
 
+import collections
 import itertools
+import warnings
 
 from factory import utils
 
@@ -100,7 +102,11 @@ def deepgetattr(obj, name, default=_UNSPECIFIED):
 class SelfAttribute(OrderedDeclaration):
     """Specific OrderedDeclaration copying values from other fields.
 
+    If the field name starts with two dots or more, the lookup will be anchored
+    in the related 'parent'.
+
     Attributes:
+        depth (int): the number of steps to go up in the containers chain
         attribute_name (str): the name of the attribute to copy.
         default (object): the default value to use if the attribute doesn't
             exist.
@@ -108,11 +114,20 @@ class SelfAttribute(OrderedDeclaration):
 
     def __init__(self, attribute_name, default=_UNSPECIFIED, *args, **kwargs):
         super(SelfAttribute, self).__init__(*args, **kwargs)
+        depth = len(attribute_name) -  len(attribute_name.lstrip('.'))
+        attribute_name = attribute_name[depth:]
+
+        self.depth = depth
         self.attribute_name = attribute_name
         self.default = default
 
     def evaluate(self, sequence, obj, containers=()):
-        return deepgetattr(obj, self.attribute_name, self.default)
+        if self.depth > 1:
+            # Fetching from a parent
+            target = containers[self.depth - 2]
+        else:
+            target = obj
+        return deepgetattr(target, self.attribute_name, self.default)
 
 
 class Iterator(OrderedDeclaration):
@@ -122,14 +137,23 @@ class Iterator(OrderedDeclaration):
 
     Attributes:
         iterator (iterable): the iterator whose value should be used.
+        getter (callable or None): a function to parse returned values
     """
 
-    def __init__(self, iterator):
+    def __init__(self, iterator, cycle=True, getter=None):
         super(Iterator, self).__init__()
-        self.iterator = iter(iterator)
+        self.getter = getter
+
+        if cycle:
+            self.iterator = itertools.cycle(iterator)
+        else:
+            self.iterator = iter(iterator)
 
     def evaluate(self, sequence, obj, containers=()):
-        return self.iterator.next()
+        value = next(self.iterator)
+        if self.getter is None:
+            return value
+        return self.getter(value)
 
 
 class InfiniteIterator(Iterator):
@@ -140,7 +164,11 @@ class InfiniteIterator(Iterator):
     """
 
     def __init__(self, iterator):
-        return super(InfiniteIterator, self).__init__(itertools.cycle(iterator))
+        warnings.warn(
+            "factory.InfiniteIterator is deprecated, and will be removed in the "
+            "future. Please use factory.Iterator instead.",
+            PendingDeprecationWarning, 2)
+        return super(InfiniteIterator, self).__init__(iterator, cycle=True)
 
 
 class Sequence(OrderedDeclaration):
@@ -250,7 +278,7 @@ class ParameteredAttribute(OrderedDeclaration):
 
         return self.generate(create, defaults)
 
-    def generate(self, create, params):
+    def generate(self, create, params):  # pragma: no cover
         """Actually generate the related attribute.
 
         Args:
@@ -276,10 +304,24 @@ class SubFactory(ParameteredAttribute):
 
     def __init__(self, factory, **kwargs):
         super(SubFactory, self).__init__(**kwargs)
-        self.factory = factory
+        if isinstance(factory, type):
+            self.factory = factory
+            self.factory_module = self.factory_name = ''
+        else:
+            # Must be a string
+            if not isinstance(factory, basestring) or '.' not in factory:
+                raise ValueError(
+                        "The argument of a SubFactory must be either a class "
+                        "or the fully qualified path to a Factory class; got "
+                        "%r instead." % factory)
+            self.factory = None
+            self.factory_module, self.factory_name = factory.rsplit('.', 1)
 
     def get_factory(self):
         """Retrieve the wrapped factory.Factory subclass."""
+        if self.factory is None:
+            # Must be a module path
+            self.factory = utils.import_object(self.factory_module, self.factory_name)
         return self.factory
 
     def generate(self, create, params):
@@ -301,22 +343,15 @@ class SubFactory(ParameteredAttribute):
 class CircularSubFactory(SubFactory):
     """Use to solve circular dependencies issues."""
     def __init__(self, module_name, factory_name, **kwargs):
-        super(CircularSubFactory, self).__init__(None, **kwargs)
-        self.module_name = module_name
-        self.factory_name = factory_name
+        factory = '%s.%s' % (module_name, factory_name)
+        warnings.warn(
+            "factory.CircularSubFactory is deprecated and will be removed in "
+            "the future. "
+            "Please replace factory.CircularSubFactory('module', 'symbol') "
+            "with factory.SubFactory('module.symbol').",
+            PendingDeprecationWarning, 2)
 
-    def get_factory(self):
-        """Retrieve the factory.Factory subclass.
-
-        Its value is cached in the 'factory' attribute, and retrieved through
-        the factory_getter callable.
-        """
-        if self.factory is None:
-            factory_class = utils.import_object(
-                self.module_name, self.factory_name)
-
-            self.factory = factory_class
-        return self.factory
+        super(CircularSubFactory, self).__init__(factory, **kwargs)
 
 
 class PostGenerationDeclaration(object):
@@ -329,6 +364,11 @@ class PostGenerationDeclaration(object):
     """
 
     def __init__(self, extract_prefix=None):
+        if extract_prefix:
+            warnings.warn(
+                "The extract_prefix argument to PostGeneration declarations "
+                "is deprecated and will be removed in the future.",
+                PendingDeprecationWarning, 3)
         self.extract_prefix = extract_prefix
 
     def extract(self, name, attrs):
@@ -352,7 +392,7 @@ class PostGenerationDeclaration(object):
         kwargs = utils.extract_dict(extract_prefix, attrs)
         return extracted, kwargs
 
-    def call(self, obj, create, extracted=None, **kwargs):
+    def call(self, obj, create, extracted=None, **kwargs):  # pragma: no cover
         """Call this hook; no return value is expected.
 
         Args:
@@ -373,13 +413,25 @@ class PostGeneration(PostGenerationDeclaration):
         self.function = function
 
     def call(self, obj, create, extracted=None, **kwargs):
-        self.function(obj, create, extracted, **kwargs)
+        return self.function(obj, create, extracted, **kwargs)
 
 
-def post_generation(extract_prefix=None):
-    def decorator(fun):
-        return PostGeneration(fun, extract_prefix=extract_prefix)
-    return decorator
+def post_generation(*args, **kwargs):
+    assert len(args) + len(kwargs) <= 1, "post_generation takes at most one argument."
+    if args and callable(args[0]):
+        # Called as @post_generation applied to a function
+        return PostGeneration(args[0])
+    else:
+        warnings.warn(
+            "The @post_generation should now be applied directly to the "
+            "function, without parameters. The @post_generation() and "
+            "@post_generation(extract_prefix='xxx') syntaxes are deprecated "
+            "and will be removed in the future; use @post_generation instead.",
+            PendingDeprecationWarning, 2)
+        extract_prefix = kwargs.get('extract_prefix')
+        def decorator(fun):
+            return PostGeneration(fun, extract_prefix=extract_prefix)
+        return decorator
 
 
 class RelatedFactory(PostGenerationDeclaration):
@@ -394,16 +446,37 @@ class RelatedFactory(PostGenerationDeclaration):
 
     def __init__(self, factory, name='', **defaults):
         super(RelatedFactory, self).__init__(extract_prefix=None)
-        self.factory = factory
         self.name = name
         self.defaults = defaults
 
+        if isinstance(factory, type):
+            self.factory = factory
+            self.factory_module = self.factory_name = ''
+        else:
+            # Must be a string
+            if not isinstance(factory, basestring) or '.' not in factory:
+                raise ValueError(
+                        "The argument of a SubFactory must be either a class "
+                        "or the fully qualified path to a Factory class; got "
+                        "%r instead." % factory)
+            self.factory = None
+            self.factory_module, self.factory_name = factory.rsplit('.', 1)
+
+    def get_factory(self):
+        """Retrieve the wrapped factory.Factory subclass."""
+        if self.factory is None:
+            # Must be a module path
+            self.factory = utils.import_object(self.factory_module, self.factory_name)
+        return self.factory
+
     def call(self, obj, create, extracted=None, **kwargs):
         passed_kwargs = dict(self.defaults)
         passed_kwargs.update(kwargs)
         if self.name:
             passed_kwargs[self.name] = obj
-        self.factory.simple_generate(create, **passed_kwargs)
+
+        factory = self.get_factory()
+        factory.simple_generate(create, **passed_kwargs)
 
 
 class PostGenerationMethodCall(PostGenerationDeclaration):
@@ -419,17 +492,27 @@ class PostGenerationMethodCall(PostGenerationDeclaration):
             ...
             password = factory.PostGenerationMethodCall('set_password', password='')
     """
-    def __init__(self, method_name, extract_prefix=None, *args, **kwargs):
+    def __init__(self, method_name, *args, **kwargs):
+        extract_prefix = kwargs.pop('extract_prefix', None)
         super(PostGenerationMethodCall, self).__init__(extract_prefix)
         self.method_name = method_name
         self.method_args = args
         self.method_kwargs = kwargs
 
     def call(self, obj, create, extracted=None, **kwargs):
+        if extracted is None:
+            passed_args = self.method_args
+
+        elif len(self.method_args) <= 1:
+            # Max one argument expected
+            passed_args = (extracted,)
+        else:
+            passed_args = tuple(extracted)
+
         passed_kwargs = dict(self.method_kwargs)
         passed_kwargs.update(kwargs)
         method = getattr(obj, self.method_name)
-        method(*self.method_args, **passed_kwargs)
+        method(*passed_args, **passed_kwargs)
 
 
 # Decorators... in case lambdas don't cut it
@@ -443,6 +526,10 @@ def iterator(func):
 
 def infinite_iterator(func):
     """Turn a generator function into an infinite iterator attribute."""
+    warnings.warn(
+        "@factory.infinite_iterator is deprecated and will be removed in the "
+        "future. Please use @factory.iterator instead.",
+        PendingDeprecationWarning, 2)
     return InfiniteIterator(func())
 
 def sequence(func):
