@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2011 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,17 @@
 
 """Compatibility tools for tests"""
 
+import sys
+
+is_python2 = (sys.version_info[0] == 2)
+
 try:
     import unittest2 as unittest
 except ImportError:
     import unittest
+
+if is_python2:
+    import mock
+else:
+    from unittest import mock
+
