@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2011 Raphaël Barrois
+# Copyright (c) 2011-2013 Raphaël Barrois
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,9 +21,15 @@
 """Tests using factory."""
 
 
+import functools
+import os
+import sys
+import warnings
+
 import factory
 
-from .compat import unittest
+from .compat import is_python2, unittest
+from . import tools
 
 
 class TestObject(object):
@@ -34,21 +40,37 @@ class TestObject(object):
         self.four = four
         self.five = five
 
-class FakeDjangoModel(object):
-    class FakeDjangoManager(object):
-        def create(self, **kwargs):
-            fake_model = FakeDjangoModel(**kwargs)
-            fake_model.id = 1
-            return fake_model
 
-    objects = FakeDjangoManager()
+class FakeModel(object):
+    @classmethod
+    def create(cls, **kwargs):
+        instance = cls(**kwargs)
+        instance.id = 1
+        return instance
+
+    class FakeModelManager(object):
+        def create(self, **kwargs):
+            instance = FakeModel.create(**kwargs)
+            instance.id = 2
+            return instance
+
+    objects = FakeModelManager()
 
     def __init__(self, **kwargs):
-        for name, value in kwargs.iteritems():
+        for name, value in kwargs.items():
             setattr(self, name, value)
             self.id = None
 
-class TestModel(FakeDjangoModel):
+
+class FakeModelFactory(factory.Factory):
+    ABSTRACT_FACTORY = True
+
+    @classmethod
+    def _create(cls, target_class, *args, **kwargs):
+        return target_class.create(**kwargs)
+
+
+class TestModel(FakeModel):
     pass
 
 
@@ -82,19 +104,21 @@ class SimpleBuildTestCase(unittest.TestCase):
             self.assertEqual(obj.three, 3)
             self.assertEqual(obj.four, None)
 
+    @tools.disable_warnings
     def test_create(self):
-        obj = factory.create(FakeDjangoModel, foo='bar')
-        self.assertEqual(obj.id, 1)
+        obj = factory.create(FakeModel, foo='bar')
+        self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
+    @tools.disable_warnings
     def test_create_batch(self):
-        objs = factory.create_batch(FakeDjangoModel, 4, foo='bar')
+        objs = factory.create_batch(FakeModel, 4, foo='bar')
 
         self.assertEqual(4, len(objs))
         self.assertEqual(4, len(set(objs)))
 
         for obj in objs:
-            self.assertEqual(obj.id, 1)
+            self.assertEqual(obj.id, 2)
             self.assertEqual(obj.foo, 'bar')
 
     def test_stub(self):
@@ -103,7 +127,7 @@ class SimpleBuildTestCase(unittest.TestCase):
         self.assertFalse(hasattr(obj, 'two'))
 
     def test_stub_batch(self):
-        objs = factory.stub_batch(FakeDjangoModel, 4, foo='bar')
+        objs = factory.stub_batch(FakeModel, 4, foo='bar')
 
         self.assertEqual(4, len(objs))
         self.assertEqual(4, len(set(objs)))
@@ -113,22 +137,23 @@ class SimpleBuildTestCase(unittest.TestCase):
             self.assertEqual(obj.foo, 'bar')
 
     def test_generate_build(self):
-        obj = factory.generate(FakeDjangoModel, factory.BUILD_STRATEGY, foo='bar')
+        obj = factory.generate(FakeModel, factory.BUILD_STRATEGY, foo='bar')
         self.assertEqual(obj.id, None)
         self.assertEqual(obj.foo, 'bar')
 
+    @tools.disable_warnings
     def test_generate_create(self):
-        obj = factory.generate(FakeDjangoModel, factory.CREATE_STRATEGY, foo='bar')
-        self.assertEqual(obj.id, 1)
+        obj = factory.generate(FakeModel, factory.CREATE_STRATEGY, foo='bar')
+        self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
     def test_generate_stub(self):
-        obj = factory.generate(FakeDjangoModel, factory.STUB_STRATEGY, foo='bar')
+        obj = factory.generate(FakeModel, factory.STUB_STRATEGY, foo='bar')
         self.assertFalse(hasattr(obj, 'id'))
         self.assertEqual(obj.foo, 'bar')
 
     def test_generate_batch_build(self):
-        objs = factory.generate_batch(FakeDjangoModel, factory.BUILD_STRATEGY, 20, foo='bar')
+        objs = factory.generate_batch(FakeModel, factory.BUILD_STRATEGY, 20, foo='bar')
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
@@ -137,18 +162,19 @@ class SimpleBuildTestCase(unittest.TestCase):
             self.assertEqual(obj.id, None)
             self.assertEqual(obj.foo, 'bar')
 
+    @tools.disable_warnings
     def test_generate_batch_create(self):
-        objs = factory.generate_batch(FakeDjangoModel, factory.CREATE_STRATEGY, 20, foo='bar')
+        objs = factory.generate_batch(FakeModel, factory.CREATE_STRATEGY, 20, foo='bar')
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
 
         for obj in objs:
-            self.assertEqual(obj.id, 1)
+            self.assertEqual(obj.id, 2)
             self.assertEqual(obj.foo, 'bar')
 
     def test_generate_batch_stub(self):
-        objs = factory.generate_batch(FakeDjangoModel, factory.STUB_STRATEGY, 20, foo='bar')
+        objs = factory.generate_batch(FakeModel, factory.STUB_STRATEGY, 20, foo='bar')
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
@@ -158,17 +184,18 @@ class SimpleBuildTestCase(unittest.TestCase):
             self.assertEqual(obj.foo, 'bar')
 
     def test_simple_generate_build(self):
-        obj = factory.simple_generate(FakeDjangoModel, False, foo='bar')
+        obj = factory.simple_generate(FakeModel, False, foo='bar')
         self.assertEqual(obj.id, None)
         self.assertEqual(obj.foo, 'bar')
 
+    @tools.disable_warnings
     def test_simple_generate_create(self):
-        obj = factory.simple_generate(FakeDjangoModel, True, foo='bar')
-        self.assertEqual(obj.id, 1)
+        obj = factory.simple_generate(FakeModel, True, foo='bar')
+        self.assertEqual(obj.id, 2)
         self.assertEqual(obj.foo, 'bar')
 
     def test_simple_generate_batch_build(self):
-        objs = factory.simple_generate_batch(FakeDjangoModel, False, 20, foo='bar')
+        objs = factory.simple_generate_batch(FakeModel, False, 20, foo='bar')
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
@@ -177,14 +204,15 @@ class SimpleBuildTestCase(unittest.TestCase):
             self.assertEqual(obj.id, None)
             self.assertEqual(obj.foo, 'bar')
 
+    @tools.disable_warnings
     def test_simple_generate_batch_create(self):
-        objs = factory.simple_generate_batch(FakeDjangoModel, True, 20, foo='bar')
+        objs = factory.simple_generate_batch(FakeModel, True, 20, foo='bar')
 
         self.assertEqual(20, len(objs))
         self.assertEqual(20, len(set(objs)))
 
         for obj in objs:
-            self.assertEqual(obj.id, 1)
+            self.assertEqual(obj.id, 2)
             self.assertEqual(obj.foo, 'bar')
 
     def test_make_factory(self):
@@ -206,11 +234,23 @@ class SimpleBuildTestCase(unittest.TestCase):
 class UsingFactoryTestCase(unittest.TestCase):
     def testAttribute(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = 'one'
 
         test_object = TestObjectFactory.build()
         self.assertEqual(test_object.one, 'one')
 
+    def test_inheritance(self):
+        @factory.use_strategy(factory.BUILD_STRATEGY)
+        class TestObjectFactory(factory.Factory, TestObject):
+            FACTORY_FOR = TestObject
+
+            one = 'one'
+
+        test_object = TestObjectFactory()
+        self.assertEqual(test_object.one, 'one')
+
     def test_abstract(self):
         class SomeAbstractFactory(factory.Factory):
             ABSTRACT_FACTORY = True
@@ -224,6 +264,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testSequence(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = factory.Sequence(lambda n: 'one' + n)
             two = factory.Sequence(lambda n: 'two' + n)
 
@@ -237,6 +279,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testSequenceCustomBegin(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             @classmethod
             def _setup_next_sequence(cls):
                 return 42
@@ -252,8 +296,43 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual('one43', test_object1.one)
         self.assertEqual('two43', test_object1.two)
 
+    def test_custom_create(self):
+        class TestModelFactory(factory.Factory):
+            FACTORY_FOR = TestModel
+
+            two = 2
+
+            @classmethod
+            def _create(cls, target_class, *args, **kwargs):
+                obj = target_class.create(**kwargs)
+                obj.properly_created = True
+                return obj
+
+        obj = TestModelFactory.create(one=1)
+        self.assertEqual(1, obj.one)
+        self.assertEqual(2, obj.two)
+        self.assertEqual(1, obj.id)
+        self.assertTrue(obj.properly_created)
+
+    def test_non_django_create(self):
+        class NonDjango(object):
+            def __init__(self, x, y=2):
+                self.x = x
+                self.y = y
+
+        class NonDjangoFactory(factory.Factory):
+            FACTORY_FOR = NonDjango
+
+            x = 3
+
+        obj = NonDjangoFactory.create()
+        self.assertEqual(3, obj.x)
+        self.assertEqual(2, obj.y)
+
     def test_sequence_batch(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = factory.Sequence(lambda n: 'one' + n)
             two = factory.Sequence(lambda n: 'two' + n)
 
@@ -267,6 +346,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testLazyAttribute(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = factory.LazyAttribute(lambda a: 'abc' )
             two = factory.LazyAttribute(lambda a: a.one + ' xyz')
 
@@ -276,6 +357,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testLazyAttributeSequence(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = factory.LazyAttributeSequence(lambda a, n: 'abc' + n)
             two = factory.LazyAttributeSequence(lambda a, n: a.one + ' xyz' + n)
 
@@ -289,6 +372,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testLazyAttributeDecorator(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             @factory.lazy_attribute
             def one(a):
                 return 'one'
@@ -301,6 +386,8 @@ class UsingFactoryTestCase(unittest.TestCase):
             n = 3
 
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = 'xx'
             two = factory.SelfAttribute('one')
             three = TmpObj()
@@ -313,8 +400,27 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(3, test_object.four)
         self.assertEqual(5, test_object.five)
 
+    def testSelfAttributeParent(self):
+        class TestModel2(FakeModel):
+            pass
+
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+            one = 3
+            three = factory.SelfAttribute('..bar')
+
+        class TestModel2Factory(FakeModelFactory):
+            FACTORY_FOR = TestModel2
+            bar = 4
+            two = factory.SubFactory(TestModelFactory, one=1)
+
+        test_model = TestModel2Factory()
+        self.assertEqual(4, test_model.two.three)
+
     def testSequenceDecorator(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             @factory.sequence
             def one(n):
                 return 'one' + n
@@ -324,6 +430,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testLazyAttributeSequenceDecorator(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             @factory.lazy_attribute_sequence
             def one(a, n):
                 return 'one' + n
@@ -337,6 +445,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testBuildWithParameters(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = factory.Sequence(lambda n: 'one' + n)
             two = factory.Sequence(lambda n: 'two' + n)
 
@@ -350,7 +460,9 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(test_object1.two, 'two1')
 
     def testCreate(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         test_model = TestModelFactory.create()
@@ -358,7 +470,9 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertTrue(test_model.id)
 
     def test_create_batch(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         objs = TestModelFactory.create_batch(20, two=factory.Sequence(int))
@@ -372,7 +486,9 @@ class UsingFactoryTestCase(unittest.TestCase):
             self.assertTrue(obj.id)
 
     def test_generate_build(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         test_model = TestModelFactory.generate(factory.BUILD_STRATEGY)
@@ -380,7 +496,9 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertFalse(test_model.id)
 
     def test_generate_create(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         test_model = TestModelFactory.generate(factory.CREATE_STRATEGY)
@@ -388,7 +506,9 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertTrue(test_model.id)
 
     def test_generate_stub(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         test_model = TestModelFactory.generate(factory.STUB_STRATEGY)
@@ -396,7 +516,9 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertFalse(hasattr(test_model, 'id'))
 
     def test_generate_batch_build(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         objs = TestModelFactory.generate_batch(factory.BUILD_STRATEGY, 20, two='two')
@@ -410,7 +532,9 @@ class UsingFactoryTestCase(unittest.TestCase):
             self.assertFalse(obj.id)
 
     def test_generate_batch_create(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         objs = TestModelFactory.generate_batch(factory.CREATE_STRATEGY, 20, two='two')
@@ -424,7 +548,9 @@ class UsingFactoryTestCase(unittest.TestCase):
             self.assertTrue(obj.id)
 
     def test_generate_batch_stub(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         objs = TestModelFactory.generate_batch(factory.STUB_STRATEGY, 20, two='two')
@@ -438,7 +564,9 @@ class UsingFactoryTestCase(unittest.TestCase):
             self.assertFalse(hasattr(obj, 'id'))
 
     def test_simple_generate_build(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         test_model = TestModelFactory.simple_generate(False)
@@ -446,7 +574,9 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertFalse(test_model.id)
 
     def test_simple_generate_create(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         test_model = TestModelFactory.simple_generate(True)
@@ -454,7 +584,9 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertTrue(test_model.id)
 
     def test_simple_generate_batch_build(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         objs = TestModelFactory.simple_generate_batch(False, 20, two='two')
@@ -468,7 +600,9 @@ class UsingFactoryTestCase(unittest.TestCase):
             self.assertFalse(obj.id)
 
     def test_simple_generate_batch_create(self):
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
             one = 'one'
 
         objs = TestModelFactory.simple_generate_batch(True, 20, two='two')
@@ -483,6 +617,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def test_stub_batch(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = 'one'
             two = factory.LazyAttribute(lambda a: a.one + ' two')
             three = factory.Sequence(lambda n: int(n))
@@ -500,6 +636,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testInheritance(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = 'one'
             two = factory.LazyAttribute(lambda a: a.one + ' two')
 
@@ -520,6 +658,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testInheritanceWithInheritedClass(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = 'one'
             two = factory.LazyAttribute(lambda a: a.one + ' two')
 
@@ -535,6 +675,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testDualInheritance(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = 'one'
 
         class TestOtherFactory(factory.Factory):
@@ -551,12 +693,26 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual('three', obj.three)
         self.assertEqual('four', obj.four)
 
+    @tools.disable_warnings
+    def test_set_building_function(self):
+        def building_function(class_to_create, **kwargs):
+            return "This doesn't even return an instance of {0}".format(class_to_create.__name__)
+
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+
+        TestModelFactory.set_building_function(building_function)
+
+        test_object = TestModelFactory.build()
+        self.assertEqual(test_object, "This doesn't even return an instance of TestModel")
+
+    @tools.disable_warnings
     def testSetCreationFunction(self):
         def creation_function(class_to_create, **kwargs):
             return "This doesn't even return an instance of {0}".format(class_to_create.__name__)
 
-        class TestModelFactory(factory.Factory):
-            pass
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
 
         TestModelFactory.set_creation_function(creation_function)
 
@@ -565,6 +721,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testClassMethodAccessible(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             @classmethod
             def alt_create(cls, **kwargs):
                 return kwargs
@@ -573,6 +731,8 @@ class UsingFactoryTestCase(unittest.TestCase):
 
     def testStaticMethodAccessible(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             @staticmethod
             def alt_create(**kwargs):
                 return kwargs
@@ -580,16 +740,87 @@ class UsingFactoryTestCase(unittest.TestCase):
         self.assertEqual(TestObjectFactory.alt_create(foo=1), {"foo": 1})
 
 
+class NonKwargParametersTestCase(unittest.TestCase):
+    def test_build(self):
+        class TestObject(object):
+            def __init__(self, *args, **kwargs):
+                self.args = args
+                self.kwargs = kwargs
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            FACTORY_ARG_PARAMETERS = ('one', 'two',)
+
+            one = 1
+            two = 2
+            three = 3
+
+        obj = TestObjectFactory.build()
+        self.assertEqual((1, 2), obj.args)
+        self.assertEqual({'three': 3}, obj.kwargs)
+
+    def test_create(self):
+        class TestObject(object):
+            def __init__(self, *args, **kwargs):
+                self.args = None
+                self.kwargs = None
+
+            @classmethod
+            def create(cls, *args, **kwargs):
+                inst = cls()
+                inst.args = args
+                inst.kwargs = kwargs
+                return inst
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            FACTORY_ARG_PARAMETERS = ('one', 'two')
+
+            one = 1
+            two = 2
+            three = 3
+
+            @classmethod
+            def _create(cls, target_class, *args, **kwargs):
+                return target_class.create(*args, **kwargs)
+
+        obj = TestObjectFactory.create()
+        self.assertEqual((1, 2), obj.args)
+        self.assertEqual({'three': 3}, obj.kwargs)
+
+
+class KwargAdjustTestCase(unittest.TestCase):
+    """Tests for the _adjust_kwargs method."""
+
+    def test_build(self):
+        class TestObject(object):
+            def __init__(self, *args, **kwargs):
+                self.args = args
+                self.kwargs = kwargs
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
+            @classmethod
+            def _adjust_kwargs(cls, **kwargs):
+                kwargs['foo'] = len(kwargs)
+                return kwargs
+
+        obj = TestObjectFactory.build(x=1, y=2, z=3)
+        self.assertEqual({'x': 1, 'y': 2, 'z': 3, 'foo': 3}, obj.kwargs)
+        self.assertEqual((), obj.args)
+
+
 class SubFactoryTestCase(unittest.TestCase):
     def testSubFactory(self):
-        class TestModel2(FakeDjangoModel):
+        class TestModel2(FakeModel):
             pass
 
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
             FACTORY_FOR = TestModel
             one = 3
 
-        class TestModel2Factory(factory.Factory):
+        class TestModel2Factory(FakeModelFactory):
             FACTORY_FOR = TestModel2
             two = factory.SubFactory(TestModelFactory, one=1)
 
@@ -599,13 +830,13 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(1, test_model.two.id)
 
     def testSubFactoryWithLazyFields(self):
-        class TestModel2(FakeDjangoModel):
+        class TestModel2(FakeModel):
             pass
 
-        class TestModelFactory(factory.Factory):
+        class TestModelFactory(FakeModelFactory):
             FACTORY_FOR = TestModel
 
-        class TestModel2Factory(factory.Factory):
+        class TestModel2Factory(FakeModelFactory):
             FACTORY_FOR = TestModel2
             two = factory.SubFactory(TestModelFactory,
                                           one=factory.Sequence(lambda n: 'x%sx' % n),
@@ -619,7 +850,7 @@ class SubFactoryTestCase(unittest.TestCase):
     def testSubFactoryAndSequence(self):
         class TestObject(object):
             def __init__(self, **kwargs):
-                for k, v in kwargs.iteritems():
+                for k, v in kwargs.items():
                     setattr(self, k, v)
 
         class TestObjectFactory(factory.Factory):
@@ -640,7 +871,7 @@ class SubFactoryTestCase(unittest.TestCase):
     def testSubFactoryOverriding(self):
         class TestObject(object):
             def __init__(self, **kwargs):
-                for k, v in kwargs.iteritems():
+                for k, v in kwargs.items():
                     setattr(self, k, v)
 
         class TestObjectFactory(factory.Factory):
@@ -649,7 +880,7 @@ class SubFactoryTestCase(unittest.TestCase):
 
         class OtherTestObject(object):
             def __init__(self, **kwargs):
-                for k, v in kwargs.iteritems():
+                for k, v in kwargs.items():
                     setattr(self, k, v)
 
         class WrappingTestObjectFactory(factory.Factory):
@@ -669,7 +900,7 @@ class SubFactoryTestCase(unittest.TestCase):
 
         class TestObject(object):
             def __init__(self, **kwargs):
-                for k, v in kwargs.iteritems():
+                for k, v in kwargs.items():
                     setattr(self, k, v)
 
         class TestObjectFactory(factory.Factory):
@@ -695,7 +926,7 @@ class SubFactoryTestCase(unittest.TestCase):
 
         class TestObject(object):
             def __init__(self, **kwargs):
-                for k, v in kwargs.iteritems():
+                for k, v in kwargs.items():
                     setattr(self, k, v)
 
         class TestObjectFactory(factory.Factory):
@@ -722,7 +953,7 @@ class SubFactoryTestCase(unittest.TestCase):
         """Test inheriting from a factory with subfactories, overriding."""
         class TestObject(object):
             def __init__(self, **kwargs):
-                for k, v in kwargs.iteritems():
+                for k, v in kwargs.items():
                     setattr(self, k, v)
 
         class TestObjectFactory(factory.Factory):
@@ -797,38 +1028,136 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(outer.side_a.inner_from_a.a, outer.foo * 2)
         self.assertEqual(outer.side_a.inner_from_a.b, 4)
 
+    def test_nonstrict_container_attribute(self):
+        class TestModel2(FakeModel):
+            pass
+
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+            one = 3
+            two = factory.ContainerAttribute(lambda obj, containers: len(containers or []), strict=False)
+
+        class TestModel2Factory(FakeModelFactory):
+            FACTORY_FOR = TestModel2
+            one = 1
+            two = factory.SubFactory(TestModelFactory, one=1)
+
+        obj = TestModel2Factory.build()
+        self.assertEqual(1, obj.one)
+        self.assertEqual(1, obj.two.one)
+        self.assertEqual(1, obj.two.two)
+
+        obj = TestModelFactory()
+        self.assertEqual(3, obj.one)
+        self.assertEqual(0, obj.two)
+
+    def test_strict_container_attribute(self):
+        class TestModel2(FakeModel):
+            pass
+
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+            one = 3
+            two = factory.ContainerAttribute(lambda obj, containers: len(containers or []), strict=True)
+
+        class TestModel2Factory(FakeModelFactory):
+            FACTORY_FOR = TestModel2
+            one = 1
+            two = factory.SubFactory(TestModelFactory, one=1)
+
+        obj = TestModel2Factory.build()
+        self.assertEqual(1, obj.one)
+        self.assertEqual(1, obj.two.one)
+        self.assertEqual(1, obj.two.two)
+
+        self.assertRaises(TypeError, TestModelFactory.build)
+
+    def test_function_container_attribute(self):
+        class TestModel2(FakeModel):
+            pass
+
+        class TestModelFactory(FakeModelFactory):
+            FACTORY_FOR = TestModel
+            one = 3
+
+            @factory.container_attribute
+            def two(self, containers):
+                if containers:
+                    return len(containers)
+                return 42
+
+        class TestModel2Factory(FakeModelFactory):
+            FACTORY_FOR = TestModel2
+            one = 1
+            two = factory.SubFactory(TestModelFactory, one=1)
+
+        obj = TestModel2Factory.build()
+        self.assertEqual(1, obj.one)
+        self.assertEqual(1, obj.two.one)
+        self.assertEqual(1, obj.two.two)
+
+        obj = TestModelFactory()
+        self.assertEqual(3, obj.one)
+        self.assertEqual(42, obj.two)
+
 
 class IteratorTestCase(unittest.TestCase):
 
     def test_iterator(self):
         class TestObjectFactory(factory.Factory):
-            one = factory.Iterator(xrange(10, 30))
+            FACTORY_FOR = TestObject
+
+            one = factory.Iterator(range(10, 30))
 
         objs = TestObjectFactory.build_batch(20)
 
         for i, obj in enumerate(objs):
             self.assertEqual(i + 10, obj.one)
 
+    def test_infinite_iterator_deprecated(self):
+        with warnings.catch_warnings(record=True) as w:
+            __warningregistry__.clear()
+
+            warnings.simplefilter('always')
+            class TestObjectFactory(factory.Factory):
+                FACTORY_FOR = TestObject
+
+                foo = factory.InfiniteIterator(range(5))
+
+            self.assertEqual(1, len(w))
+            self.assertIn('InfiniteIterator', str(w[0].message))
+            self.assertIn('deprecated', str(w[0].message))
+
+    @tools.disable_warnings
     def test_infinite_iterator(self):
         class TestObjectFactory(factory.Factory):
-            one = factory.InfiniteIterator(xrange(5))
+            FACTORY_FOR = TestObject
+
+            one = factory.InfiniteIterator(range(5))
 
         objs = TestObjectFactory.build_batch(20)
 
         for i, obj in enumerate(objs):
             self.assertEqual(i % 5, obj.one)
 
-    def test_infinite_iterator_list_comprehension(self):
+    @unittest.skipUnless(is_python2, "Scope bleeding fixed in Python3+")
+    @tools.disable_warnings
+    def test_infinite_iterator_list_comprehension_scope_bleeding(self):
         class TestObjectFactory(factory.Factory):
-            one = factory.InfiniteIterator([j * 3 for j in xrange(5)])
+            FACTORY_FOR = TestObject
+
+            one = factory.InfiniteIterator([j * 3 for j in range(5)])
 
         # Scope bleeding: j will end up in TestObjectFactory's scope.
 
         self.assertRaises(TypeError, TestObjectFactory.build)
 
+    @tools.disable_warnings
     def test_infinite_iterator_list_comprehension_protected(self):
         class TestObjectFactory(factory.Factory):
-            one = factory.InfiniteIterator([_j * 3 for _j in xrange(5)])
+            FACTORY_FOR = TestObject
+
+            one = factory.InfiniteIterator([_j * 3 for _j in range(5)])
 
         # Scope bleeding : _j will end up in TestObjectFactory's scope.
         # But factory_boy ignores it, as a protected variable.
@@ -839,9 +1168,11 @@ class IteratorTestCase(unittest.TestCase):
 
     def test_iterator_decorator(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             @factory.iterator
             def one():
-                for i in xrange(10, 50):
+                for i in range(10, 50):
                     yield i
 
         objs = TestObjectFactory.build_batch(20)
@@ -849,11 +1180,30 @@ class IteratorTestCase(unittest.TestCase):
         for i, obj in enumerate(objs):
             self.assertEqual(i + 10, obj.one)
 
+    def test_infinite_iterator_decorator_deprecated(self):
+        with warnings.catch_warnings(record=True) as w:
+            __warningregistry__.clear()
+
+            warnings.simplefilter('always')
+            class TestObjectFactory(factory.Factory):
+                FACTORY_FOR = TestObject
+
+                @factory.infinite_iterator
+                def one():
+                    return range(5)
+
+            self.assertEqual(1, len(w))
+            self.assertIn('infinite_iterator', str(w[0].message))
+            self.assertIn('deprecated', str(w[0].message))
+
+    @tools.disable_warnings
     def test_infinite_iterator_decorator(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             @factory.infinite_iterator
             def one():
-                for i in xrange(5):
+                for i in range(5):
                     yield i
 
         objs = TestObjectFactory.build_batch(20)
@@ -865,6 +1215,48 @@ class IteratorTestCase(unittest.TestCase):
 class PostGenerationTestCase(unittest.TestCase):
     def test_post_generation(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
+            one = 1
+
+            @factory.post_generation
+            def incr_one(self, _create, _increment):
+                self.one += 1
+
+        obj = TestObjectFactory.build()
+        self.assertEqual(2, obj.one)
+        self.assertFalse(hasattr(obj, 'incr_one'))
+
+        obj = TestObjectFactory.build(one=2)
+        self.assertEqual(3, obj.one)
+        self.assertFalse(hasattr(obj, 'incr_one'))
+
+    def test_post_generation_hook(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
+            one = 1
+
+            @factory.post_generation
+            def incr_one(self, _create, _increment):
+                self.one += 1
+                return 42
+
+            @classmethod
+            def _after_postgeneration(cls, obj, create, results):
+                obj.create = create
+                obj.results = results
+
+        obj = TestObjectFactory.build()
+        self.assertEqual(2, obj.one)
+        self.assertFalse(obj.create)
+        self.assertEqual({'incr_one': 42}, obj.results)
+
+    @tools.disable_warnings
+    def test_post_generation_calling(self):
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = 1
 
             @factory.post_generation()
@@ -881,9 +1273,11 @@ class PostGenerationTestCase(unittest.TestCase):
 
     def test_post_generation_extraction(self):
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             one = 1
 
-            @factory.post_generation()
+            @factory.post_generation
             def incr_one(self, _create, increment=1):
                 self.one += increment
 
@@ -904,10 +1298,40 @@ class PostGenerationTestCase(unittest.TestCase):
             self.assertEqual(kwargs, {'foo': 13})
 
         class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+
             bar = factory.PostGeneration(my_lambda)
 
         obj = TestObjectFactory.build(bar=42, bar__foo=13)
 
+    def test_post_generation_method_call(self):
+        calls = []
+
+        class TestObject(object):
+            def __init__(self, one=None, two=None):
+                self.one = one
+                self.two = two
+                self.extra = None
+
+            def call(self, *args, **kwargs):
+                self.extra = (args, kwargs)
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = 3
+            two = 2
+            post_call = factory.PostGenerationMethodCall('call', one=1)
+
+        obj = TestObjectFactory.build()
+        self.assertEqual(3, obj.one)
+        self.assertEqual(2, obj.two)
+        self.assertEqual(((), {'one': 1}), obj.extra)
+
+        obj = TestObjectFactory.build(post_call__one=2, post_call__two=3)
+        self.assertEqual(3, obj.one)
+        self.assertEqual(2, obj.two)
+        self.assertEqual(((), {'one': 2, 'two': 3}), obj.extra)
+
     def test_related_factory(self):
         class TestRelatedObject(object):
             def __init__(self, obj=None, one=None, two=None):
@@ -952,6 +1376,70 @@ class PostGenerationTestCase(unittest.TestCase):
         # RelatedFactory received "parent" object
         self.assertEqual(obj, obj.related.three)
 
+    def test_related_factory_no_name(self):
+        relateds = []
+        class TestRelatedObject(object):
+            def __init__(self, obj=None, one=None, two=None):
+                relateds.append(self)
+                self.one = one
+                self.two = two
+                self.three = obj
+
+        class TestRelatedObjectFactory(factory.Factory):
+            FACTORY_FOR = TestRelatedObject
+            one = 1
+            two = factory.LazyAttribute(lambda o: o.one + 1)
+
+        class TestObjectFactory(factory.Factory):
+            FACTORY_FOR = TestObject
+            one = 3
+            two = 2
+            three = factory.RelatedFactory(TestRelatedObjectFactory)
+
+        obj = TestObjectFactory.build()
+        # Normal fields
+        self.assertEqual(3, obj.one)
+        self.assertEqual(2, obj.two)
+        # RelatedFactory was built
+        self.assertIsNone(obj.three)
+        self.assertEqual(1, len(relateds))
+        related = relateds[0]
+        self.assertEqual(1, related.one)
+        self.assertEqual(2, related.two)
+        self.assertIsNone(related.three)
+
+        obj = TestObjectFactory.build(three__one=3)
+        # Normal fields
+        self.assertEqual(3, obj.one)
+        self.assertEqual(2, obj.two)
+        # RelatedFactory was build
+        self.assertIsNone(obj.three)
+        self.assertEqual(2, len(relateds))
+
+        related = relateds[1]
+        self.assertEqual(3, related.one)
+        self.assertEqual(4, related.two)
+
+
+class CircularTestCase(unittest.TestCase):
+    def test_example(self):
+        sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
+
+        from .cyclic import foo
+        f = foo.FooFactory.build(bar__foo=None)
+        self.assertEqual(42, f.x)
+        self.assertEqual(13, f.bar.y)
+        self.assertIsNone(f.bar.foo)
+
+        from .cyclic import bar
+        b = bar.BarFactory.build(foo__bar__foo__bar=None)
+        self.assertEqual(13, b.y)
+        self.assertEqual(42, b.foo.x)
+        self.assertEqual(13, b.foo.bar.y)
+        self.assertEqual(42, b.foo.bar.foo.x)
+        self.assertIsNone(b.foo.bar.foo.bar)
+
+
 
 if __name__ == '__main__':
     unittest.main()
